# -*- coding: utf-8 -*-
# Generated by Django 1.9.11 on 2016-11-08 21:17
from __future__ import unicode_literals

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('localized_recurrence', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='localizedrecurrence',
            name='offset2',
            field=models.DurationField(default=datetime.timedelta(0)),
        ),
    ]
