# -*- coding: utf-8 -*-
# Generated by Django 1.9.11 on 2016-11-08 21:51
from __future__ import unicode_literals

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('localized_recurrence', '0003_offset_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='localizedrecurrence',
            name='offset',
        ),
        migrations.RenameField(
            model_name='localizedrecurrence',
            old_name='offset2',
            new_name='offset',
        ),
    ]
